#!/bin/bash

# functions for getting hardware lists from hal

# for localized lists set
# dec_pt=$(LC_NUMERIC="$in_language" locale decimal_point)

#note: hal returns devices in wrong order (sdb sda2 sda1 sda...)

dec_pt="."
. alterator-sh-functions

# convert bytes to human readable form:
# 1 b, 12 b, 123 b, 1.23 Kb, 12.3 Kb, 123 Kb 1.23 Mb etc.
printable_size(){
  local size=$1
  local frac=0
  local byte="`_ "b"`"
  local units=$byte
  local div=1024
  local u
  for u in "`_ "Kb"`" "`_ "Mb"`" "`_ "Gb"`" "`_ "Tb"`"; do
    if [ $(($size < 1000)) = 1 ]; then break; fi
    frac=$((($size % $div)*1000/$div))  # we want frac in the range 0..999
    size=$(($size / $div))
    units=$u
  done
  if      [ "$(($size > 99))" = 1 ]; then frac=$(($frac/100))
  else if [ "$(($size > 9))" = 1 ]; then frac=$(printf "%02d" $(($frac/10)))
  fi; fi;

  # add zeroes
  frac=$(printf "%02d" $((frac/10)))

  [ "$units" = "$byte" ] && frac="" || frac="${dec_pt:-.}$frac"
  echo "$size$frac $units"
}

# list all hard drives:
# (name "/dev/sda" label "hard drive sda, FUJITSU MHW2160B, 149 Гб")
list_hard_drives(){
  local hd_label="`_ "Hard drive"`"
  local udi
  for udi in $(get_disks); do
    [ "$(hal-get-property --udi "$udi" --key storage.removable)" = "false" ] || continue
    local info=$(hal-get-property --udi "$udi" --key storage.model)
    local size=$(hal-get-property --udi "$udi" --key storage.size)
    local  dev=$(hal-get-property --udi "$udi" --key block.device)
    size=$(printable_size $size)
    local sym=$(convert_dev2sym $dev)
    echo "$sym" "$hd_label ${dev##*/}, $info, $size"
  done
}

# list partitions
# (name "/dev/sda4" label "раздел sda4 (LVM2_member), 86,3 Гб")
# (name "/dev/sda3" label "раздел sda3 (ext3), 8,39 Гб")
# (name "/dev/sda2" label "раздел sda2 (swap), 1 Гб")
# (name "/dev/sda1" label "раздел sda1 (ntfs), 53,3 Гб")
list_partitions(){
  local part_label="`_ "partition"`"
  local udi
  for udi in $(get_partitions); do
    local   fs=$(hal-get-property --udi "$udi" --key volume.fstype)
    local size=$(hal-get-property --udi "$udi" --key volume.size)
    local  dev=$(hal-get-property --udi "$udi" --key block.device)
    size=$(printable_size $size)
    local sym=$(convert_dev2sym $dev)
    echo "$sym" "$part_label ${dev##*/} ($fs), $size"
  done
}

# find other loaders in system (on partitions only!)
# (for creation "other" sections in lilo.conf)
find_others(){

  # Exclude self partitions
  exclude="$(get_part_with_mntpt "/")
           $(get_part_with_mntpt "/boot")
           $(get_part_with_mntpt "/usr")
           $(get_part_with_mntpt "/home")
           $(get_part_with_mntpt "/var")
           "
  local udi win_n lin_n label
  for udi in $(get_partitions); do


    # hal can't get volume.partition.type for some partitions(!)
    # so let's examine fstype too
    local   dev=$(hal-get-property --udi "$udi" --key block.device)
    [ "${exclude##*$dev}" = "$exclude" ] || continue

    local fstype=$(hal-get-property --udi "$udi" --key volume.fstype)
    local   type=$(get_part_type "$udi")

    [ "$fstype" = 'swap' -o "$fstype" = '' ] && type=
    [ "$fstype" = 'ext2' -o "$fstype" = 'ext3' -o "$fstype" = 'reiserfs' ] && type='83'
    [ "$fstype" = 'ntfs' -o "$fstype" = 'vfat' -o "$fstype" = 'fat' ] && type='b'

    [ -n "$type" ] || continue

    bootsector "$dev" >&2 || continue

    case "$type" in
      1|4|6|7|b|c|e|f|11|14|16|17|1b|1c|1e|86|87|c1|c4|c6|ef)
        label="Windows${win_n:+$win_n}"
        [ -z "$win_n" ] && win_n=0
        win_n=$((win_n+1))
      ;;
      81|83|85|88|8e|f0|fd)
        label="Linux${lin_n:+$lin_n}"
        [ -z "$lin_n" ] && lin_n=0
        lin_n=$((lin_n+1))
      ;;
      2|3) label="Xenix"    ;;
      8|9) label="Aix"      ;;
      a)   label="OS/2"     ;;
      39)  label="Plan 9"   ;;
      a5)  label="FreeBSD"  ;;
      a6)  label="OpenBSD"  ;;
      a9)  label="NetBSD"   ;;
      ab)  label="Darwin"   ;;
      63)  label="GNU HURD" ;;
      be)  label="Solaris"  ;;
      eb)  label="BeOS"     ;;
      #... add more
      *)  continue ;;
    esac
    # 1. device (from /dev/disks/by-id/*)
    # 2. short label (without spaces - for lilo.conf)
    echo $(convert_dev2sym $dev) $label
  done
}

# get partition type for udi
get_part_type(){
  local type=$(hal-get-property --udi "$1" --key volume.partition.type 2>/dev/null)
  type=${type#0x}
  type=${type#0}
  echo $type
}

# get partition udi's excluding removables
get_partitions(){
  local udi
  for udi in $(hal-find-by-capability --capability volume); do
    # hal-find-by-property for bools does not work(!)
#    [ "$(hal-get-property --udi "$udi" --key volume.is_partition)" = "true" ] || continue
    # skip removables
    local parent_udi=$(hal-get-property --key info.parent --udi "$udi")
    [ "$(hal-get-property --udi "$parent_udi" --key storage.removable)" = "false" ] || continue
    # skip raid members
    [ "$(hal-get-property --udi "$udi" --key volume.fsusage)" != "raid" ] || continue
    # skip extended
    [ "$(get_part_type "$udi")" = "5" ] && continue
    local dev=$(hal-get-property --key block.device --udi "$udi")
    echo -e "$udi\t${dev%[0-9]}\t${dev##*[a-z]}"
  done | sort -k 2,2 -k3,3n | cut -f 1
}

# get partition devices
get_partition_devices(){
  local udi
  for udi in $(get_partitions); do
    convert_dev2sym $(hal-get-property --udi "$udi" --key block.device)
  done
}

# get disks udi's excluding removables
get_disks(){
  local udi
  for udi in $(hal-find-by-property --key storage.drive_type --string disk | tac); do
    # skip removables
    [ "$(hal-get-property --udi "$udi" --key storage.removable)" = "false" ] || continue
    # skip raids
    [ "$(hal-get-property --udi "$udi" --key storage.bus)" != "linux_raid" ] || continue
    echo $udi
  done
}

# get devices of partition with given mntpt
# note: Hal does not get partition mount points in installer's chroot!
#       So I get them from /proc/mounts
# note: In /proc/mounts can appear strange device names such as
#       /dev/evms/sda1  or  /dev/disk/by-uuid/...
#       Hal does not understand these names
#       So I found device using its uuid (got by /sbin/blkid)
# note: i don'n know how to work correctly with raid + evms!
#       uuid for such device can't be find in /dev/disk/by-uuid
#       and in hal

get_part_with_mntpt(){
  local dev mntpt type opts n1 n2
  while read dev mntpt type opts n1 n2; do
    if [ "$mntpt" = "$1" ]; then

      # hack with /dev/evms/md devices
      local d=/dev/${dev#/dev/evms/md/}
      if [ "$dev" != "$d" -a -b $d ]; then
        echo $d
        return
      fi

      # convert dev -> uuid -> dev
      local uuid=$(/sbin/blkid $dev -s UUID)
      uuid=${uuid#*UUID=}
      uuid=${uuid#*\"}
      uuid=${uuid%\"*}
      [ -n "$uuid" ] || continue
      hal-get-property --udi "$(hal-find-by-property --key volume.uuid --string $uuid)" --key block.device
    fi
  done < '/proc/mounts'
}

# find partition with our /boot directory:
get_boot_part(){
  local dev="$(get_part_with_mntpt "/boot") $(get_part_with_mntpt "/")"
  dev=${dev## }
  dev=${dev%% *}
  # don't protect device name by convert_dev2sym here
  echo $dev
}

# for cdrom openclose
# cdroms on ide bus only (!?)
get_ide_cdroms(){
  local cdroms=$(hal-find-by-property --key storage.drive_type --string cdrom | tac)
  local udi
  for udi in $cdroms; do
    local bus=$(hal-get-property --udi $udi --key storage.bus)
    [ "$bus" = "ide" ] || continue
    # we don't need to protect device name by convert_dev2sym
    # becouse we don't want to save this in lilo.conf
    hal-get-property --udi $udi --key block.device
  done
}

# get default place for bootloader:
get_default_boot_dev(){
  if [ -n "$raidboot" ]; then
    convert_dev2sym "$raidboot"
    return 0
  fi
  local disk=$(get_disks | head -n1)
  local dev=$(hal-get-property --udi "$disk" --key block.device)
  convert_dev2sym $dev
}

# if removable device exists device names can be changed from
# time to time... So let's substitute them in lilo.conf
# by symlinks from /dev/diks/by-id/

convert_dev2sym(){
  local dev=$1
  if [ -z "$dev" ]; then
    echo "use: convert_dev2sym <device>"
    return
  fi
  local ret=
  local s=
  for s in $(ls -1 /dev/disk/by-id); do
    local d=$(readlink -f /dev/disk/by-id/$s)
    if [ "$d" = "$dev" ]; then
      # don't return ata* if scsi* exists
      # it is done automatically due to sorting,
      # but let's keep explicit code here:
      [ "${ret#scsi}" != "$ret" -a "${s#ata}" != "$s" ] && continue
      ret="/dev/disk/by-id/$s"
    fi
  done
  echo ${ret:-$dev}
}
convert_sym2dev(){
  local sym=$1
  if [ -z "$sym" ]; then
    echo "use: convert_sym2dev <symlink>"
    return
  fi
  local dev=$(readlink -f $sym)
  echo ${dev:-$sym}
}

